<?php
set_time_limit(0);

class SmartResponderApi{
    protected $_smart_api_id;
    protected $_smart_api_pass;
    protected $_smart_api_delivery_id;
    protected $_smart_api_url = 'http://api.smartresponder.ru/';
    
    public function __construct($api_id, $api_pass, $delivery_id) {
        $this->_smart_api_id = $api_id;
        $this->_smart_api_pass =$api_pass;
        $this->_smart_api_delivery_id = $delivery_id;
    }

    
    /**
     * @param array , вида ключ=>значение. В массиве передавать все, что нужно для выборки, кроме  api_id, delivery_id, format, password
     * @return object 
    */      
    public function query($params, $url = 'subscribers'){
            $url =  $this->_smart_api_url . $url . '.html';
            
            $params['api_id']      = $this->_smart_api_id;
            $params['delivery_id'] = $this->_smart_api_delivery_id;
            $params['format']      = 'json';
    
            // Склейка
            $_glue = $params; 
            $_glue['password']  =  $this->_smart_api_pass;
            $glue = array();
    
            foreach($_glue as $key=>$value){
                $glue[] = "$key=$value";
            }
            $glue = implode(':', $glue);
            $md5 = md5($glue);
            
            $params['hash'] = $md5;
            
            // Готовим данные для запроса
            $data_url = array();
            foreach($params as $key=>$value){
                $value = urlencode($value);
                $data_url[] = "$key=$value";
            } 
            
            $data_url = implode('&', $data_url);  
            $data_len = mb_strlen ($data_url, 'UTF-8'); 
            
            $result = file_get_contents ($url, false, stream_context_create (array ('http'=>array ('method'=>'POST'
                , 'header'=>""
                , 'content'=>$data_url
            ))));
            
            $result = json_decode($result);

            return $result;
        } 
     
     public function get($name){
            return $this->$name;
     }        
}

class SecretPageHandler{
    
    private $_smartresponder; 
    private $_data; // array, from form  
    private $_cookieName;
    
    protected $recoveryTemplate, $secret_key; 
    
    /**
     * @param SmartResponderApi object
     * @param array
    */
    public function __construct($SmartresponderApiObject, $data, $cookieName){
        $this->_smartresponder = $SmartresponderApiObject;
        $this->_data = $data;
        $this->_cookieName = $cookieName;
    }
    
    
    public function execute(){
        $action = '_action_' . $this->_data['action'];
       
        if(method_exists($this, $action)){
            $this->_beforeExecute();
            $this->$action();
        }
        else{
            self::ajax_response('Вызванный метод неизвестен');
        }
         
    }
    
    ///// ACTIONS
    // subscribe action
    private function _action_subscribeMe(){

        $query = array(
                'action'        =>  'create',
                'email'         => $this->_data['email'],
                'first_name'    => $this->_data['name']
        );   
         
        $result = $this->_smartresponder->query($query);  

        if($result->error->code == '-2.EMAIL_IS_ALREADY_IN_YOUR_LIST' or $result->result){
            self::ajax_response('', 'success', 'step2'); 
        }
        else{
            self::ajax_response($result->error->code);    
        }
            
    }
    
    // recovery action
    private function _action_recoveryUnlock(){
        $query = array(
                'action'        => 'list',
                'search[email]' => $this->_data['email'],
        );
                
        $result = $this->_smartresponder->query($query);
        $result = $result->list->count;
        
        
        // get author && author_email
        $deliveries = $this->_smartresponder->query(array(
            'action'  => 'list'
        ),'deliveries');        
        
        if(!$deliveries->result){
            self::ajax_response('Рассылки не определены');  
        } 
        
        $delivery = $deliveries->list->elements[0];                   
        $query['from_name']    = $delivery->author_name;
        $query['from_email']   = $delivery->email_from;  
                        
        // send 
        if($result){
            $query['action']       = 'on_message';
            $query['subject']      = 'Код доступа';
            $query['text_body']    = $this->_getRecoveryTemplate();
            $query['priority']     = 2;

            $result = $this->_smartresponder->query($query, 'subscribe');   

            if(!$result->result){
                self::ajax_response('Ошибка сервиса, попробуйте еще раз');
            }   
            self::ajax_response('Код отправлен на Ваш Email', 'success', 'pushCode2');
             
        }
        else{
             self::ajax_response('', 'success', 'unknowEmail');
        }        
    }    
    
    
    private function _action_pushCode(){
        $this->_action_enterCode();
    }    


    private function _action_enterCode(){
        $code = preg_replace('/\D+/', '', $this->_data['code']);

        if($code != $this->secret_key){
            self::ajax_response('', 'error', 'enterCodeError');
        }
        else{
            $this->_unlock();
            self::ajax_response('', 'success', 'unlock');
        }
    }    
    //// END ACTIONS
    
    
    
    public function set($name, $value){
        $this->$name = $value;
    } 
    
    
    private function _beforeExecute(){
        $regex = '/^[\w.-]+(\+[\w.-]+)*@\w+[\w.-]*?\.\w{2,4}$/';
        $d = & $this->_data;

        if(isset($d['email'])){
            $d['email'] = trim($d['email']);
            
            if(!preg_match($regex, $d['email'])){
                self::ajax_response('Введите корректный email');
            }
        }
        
        if(isset($d['name'])){
            $d['name']  = preg_replace('/[^а-яa-z- ]+/ui', '', $d['name']);
            $d['name']  = trim($d['name']);
            if(!$d['name']) self::ajax_response('Введите корректное имя');
        }        
    }

    
    
    private function _getRecoveryTemplate(){
        if(!$this->recoveryTemplate){
            self::ajax_response('Шаблон не определен');
        } 
        
        $_text = explode("\n", $this->recoveryTemplate);
        $text  = array();
          
        foreach($_text as $txt){
           $txt = preg_replace("/  /", " ", $txt);
           $text[] = trim($txt); 
        }
        
        $text = implode("\n", $text);   
        
        return $text;
    } 
    
    private function _unlock(){
        $life = time() + 157680000; 
        setcookie($this->_cookieName, $this->_cookieName , $life, '/'); 
    }
    
    
    /**
     * @param string out-message
     * @param string message-type error or success
     * @param string name method from lockContent js-object
    */
    public static function ajax_response($message, $type = 'error', $callback = null){
        exit(json_encode(array($type=>$message, 'callback' => $callback)));
    } 
    
    /**
     * Проверка ключа плагина.
    */
    public static function checkKey(){
                    return true;

    }               
    
}
?>